
import numpy as np


converter = 180.0 / np.pi
        
def angle_table( x_range, y_range ):
    angle = np.zeros( (x_range, y_range), dtype=float)
    norm_table = np.zeros_like(angle)
    
    for i in range(x_range):
        x_comp = i*i
        for j in range(y_range):
            y_comp = j*j
            tmp_norm = np.sqrt(x_comp + y_comp)
            angle[i, j] = converter*np.arccos( i / tmp_norm )
            norm_table[i, j] = tmp_norm

    return norm_table, angle


if __name__ == '__main__':
    x_range = 2000
    y_range = 4000

    norm_matrix, angle_matrix = angle_table( x_range, y_range )
    np.save('../aux/norm_matrix.npy', norm_matrix)
    np.save('../aux/angle_table.npy', angle_matrix)




"""This script includes the entry point.

For more information on usage, run estare -h, or see the documentation.
"""

import numpy as np
from estare.src.init import examine, setup
from estare.src.feature import extract
from estare.src.align import align
from estare.src.scan import scan

from matplotlib import pyplot as plt
import argparse
import os
from skimage.io import imsave





def assemble(args):
    # unpack the arguments
    img_1 = args.layer_1
    img_2 = args.layer_2
    pivot_1 = args.feature_1
    pivot_2 = args.feature_2

    # align and stack the two input frames
    stacked_frame = align(img_1, img_2, pivot_1, pivot_2, save=True)

         
parser = argparse.ArgumentParser(prog='estare', description='''Program for aligning and stacking astro photos.''',
                                 epilog='''estare is a Persian word for star.''')

sub_parsers = parser.add_subparsers()

# create the parser for the feature detection command
parse_feature = sub_parsers.add_parser('scan', help='''Scan an input image to find suitable features such as (a) 
bright star(s) that can be used for aligning multiple frames.''')

parse_feature.add_argument('image', action='store', type=str,  help='The input image to be scanned.')
parse_feature.add_argument('kapa',  action='store', type=float, help='''Threshold for picking pixels 
of a certain brightness. It takes a floating point values from the range [0, 1]. Pick a larger value 
to limit the detected features to those of a higher brightness.''')
parse_feature.set_defaults(func=scan)

parse_stack = sub_parsers.add_parser('stack', help='''This mode should normally be used after running 
the scan mode whereby alignment features can be extracted. The stack mode uses the coordinates of two 
features shared in two frames to align them. It then adjusts them and stacks them up. Two input images 
along with two identical features for each image are required.''')

parse_stack.add_argument('layer_1', help='Path and name of the reference image (base layer)')
parse_stack.add_argument('layer_2', help='Path and name of the second image (top layer)')
parse_stack.add_argument('feature_1', help='''Path and name of a .npy file containing the coordinates 
of the feature in the reference layer''')
parse_stack.add_argument('feature_2', help='''Path and name of a .npy file containing the coordinates 
of the feature in the second layer which will be lined up with the reference layer.''')
parse_stack.set_defaults(func=assemble)

args = parser.parse_args()


if __name__ == '__main__':
    args = parser.parse_args()
    args.func(args)   # call the default function



"""This module should be imported by the entry point script.

It implements the function scan that is used for detecting 
features that are suitable for aligning two images.  
"""

from estare.src.init import examine, setup
from estare.src.feature import extract
from matplotlib import pyplot as plt
from skimage.io import imsave

import os
import numpy as np


def scan(args):
    """Finds features that are suitable for aligning two images.

    This function shows the input image in a graph first. It then 
    goes through every pixel of it and compares its brightness with 
    a given threshold (kapa) specified by the user. If the pixel's 
    brightness is higher than or equal to the threshold, a marker 
    will be shown on that location in the graph and the user can 
    accept it as a feature by pressing any key. The pixel can 
    alternatively be discarded by a left click.

    Args:
        args : user inputs parsed by argparse
    """
    
    # handle input arguments
    imagePath = args.image
    threshold = args.kapa

    img_as_array, x_range, y_range = examine(imagePath, save=True, verbose=False, graphics=False)
    # Replace with:
    # img_as_array = FloatImage(imagePath)
    # img_as_array.print_info()
    # img_as_array.save()
    # x_range, y_range = img_as_array.size()
    
    print(f'Image has x-range = {x_range}, and y-range = {y_range}')

    imgGray = img_as_array @ [0.2126, 0.7152, 0.0722]  # image in grayscale
    # Replace with (makes FloatImage redundant):
    # imgGray = GrayImage(imagePath)
    # imgGray.print_info()
    # imgGray.save()
    # x_range, y_range = imgGray.size()

    fig2, frame = plt.subplots(1, 1)
    image_features = frame.imshow(imgGray, cmap='gray')
    frame.set_title('Marked features', fontsize=14)

    numFeatures, indices, markers = extract(imgGray, xRng=[0, x_range], yRng=[0, y_range],
                                            kapa=threshold)
    # Replace with:
    # numFeatures, indices, markers = imgGray.extract(xRng=[0, x_range], yRng=[0, y_range], kapa=threshold)

    print(f'''
    Total number of features detected: {numFeatures}. If this is too many, consider increasing  
    the threshold input.
    ''')

    # Loop over features and ask the user if a feature is of interest to be saved
    keepFeature = False

    xy_FeatureCount = 0   # number of already-saved feature coordinates initialized to zero
    binFeatureCount = 0   # number of already-saves feature images in binary format initialized to zero
    imgFeatureCount = 0   # number of already-saves feature images in .png format initialized to zero

    xy_RefusedCount = 0   # number of already-saved refused coordinates initialized to zero
    binRefusedCount = 0   # number of already-saves refused images in binary format initialized to zero
    imgRefusedCount = 0   # number of already-saves refused images in .png format initialized to zero  

    for files in os.listdir('./data/features'):
        if files.endswith('.png'):
            imgFeatureCount += 1

    for files in os.listdir('./data/features/pixels'):
        if files.endswith('.npy'):
            binFeatureCount += 1

    for files in os.listdir('./data/features/coordinates'):
        if files.endswith('.npy'):
            xy_FeatureCount += 1


    for files in os.listdir('./data/refuse'):
        if files.endswith('.png'):
            imgRefusedCount += 1

    for files in os.listdir('./data/refuse/pixels'):
        if files.endswith('.npy'):
            binRefusedCount += 1

    for files in os.listdir('./data/refuse/coordinates'):
        if files.endswith('.npy'):
            xy_RefusedCount += 1    
            
    selectedFeatures = 0  # counter
    refusedFeatures = 0  # counter
    
    
    # TODO: add a feature to use matplotlib.ginput
    if numFeatures > 0:
        print('''A detected feature is displayed by a black square. Press any key to save the current feature, or click 
        the mouse to discard it...''')
        for pair_0, pair_1 in zip(indices[0], indices[1]):
            if pair_0 > 3 and pair_1 > 3:
                arrow = frame.annotate('O', xy=(pair_1, pair_0), arrowprops=dict(color='lime',arrowstyle='->'))
                plt.draw()  # , plt.pause(0.01)
                btnpress = plt.waitforbuttonpress(-1)
                if btnpress:
                    selectedFeatures += 1
                    np.save('./data/features/coordinates/feature_{}'.format(xy_FeatureCount), [pair_0, pair_1])
                    xy_FeatureCount += 1
                    np.save('./data/features/pixels/feature_{}_pixels'.format(binFeatureCount), imgGray[pair_0-10:pair_0+10, pair_1-10:pair_1+10])
                    binFeatureCount += 1
                    imsave('./data/features/feature_{}_pixels.png'.format(imgFeatureCount), imgGray[pair_0-10:pair_0+10, pair_1-10:pair_1+10])
                    imgFeatureCount += 1
                    # TODO: Find install path and cd to data
                    plt.waitforbuttonpress(0.1)
                else:
                    refusedFeatures += 1
                    np.save('./data/refuse/coordinates/refused_{}'.format(xy_RefusedCount), [pair_0, pair_1])
                    xy_RefusedCount += 1
                    np.save('./data/refuse/pixels/refused_{}_pixels'.format(binRefusedCount), imgGray[pair_0-10:pair_0+10, pair_1-10:pair_1+10])
                    binRefusedCount += 1
                    imsave('./data/refuse/refused_{}_pixels.png'.format(imgRefusedCount), imgGray[pair_0-10:pair_0+10, pair_1-10:pair_1+10])
                    imgRefusedCount += 1
                arrow.remove()   # removing the arrow must be the last thing to do at the end of the if-block
                
    print('Feature detection completed.')
    plt.show()
